@@ -5,6 +5,7 @@ use std::future::Future;
 use std::marker::PhantomData;
 use std::ops::*;
 use std::pin::Pin;
+use std::sync::atomic::*;
 use std::sync::Arc;
 use std::sync::Mutex;
 use std::task::Context;
@@ -21,10 +22,16 @@ where
     fn data(&self, data: Vec<u8>);
 
     fn error(&self, error: CallError);
+
+    fn wapm(&self) -> &str;
+
+    fn topic(&self) -> &str;
 }
 
+#[derive(Debug)]
 pub struct CallState {
     pub(crate) result: Option<Result<Vec<u8>, CallError>>,
+    pub(crate) callbacks: Vec<Finish>,
 }
 
 #[derive(Derivative, Clone)]
@@ -33,29 +40,45 @@ pub struct CallState {
 pub struct Call {
     pub(crate) wapm: Cow<'static, str>,
     pub(crate) topic: Cow<'static, str>,
+    pub(crate) format: SerializationFormat,
+    pub(crate) session: Option<String>,
     pub(crate) handle: CallHandle,
     pub(crate) parent: Option<CallHandle>,
-    #[derivative(Debug = "ignore")]
     pub(crate) state: Arc<Mutex<CallState>>,
-    pub(crate) callbacks: Arc<Mutex<Vec<Finish>>>,
+    pub(crate) drop_on_data: Arc<AtomicBool>,
 }
 
 impl CallOps for Call {
     fn data(&self, data: Vec<u8>) {
-        let mut state = self.state.lock().unwrap();
-        state.result = Some(Ok(data));
+        {
+            let mut state = self.state.lock().unwrap();
+            state.result = Some(Ok(data));
+        }
+        if self
+            .drop_on_data
+            .compare_exchange(true, false, Ordering::Release, Ordering::Relaxed)
+            .is_ok()
+        {
+            super::drop(self.handle);
+            crate::engine::BusEngine::remove(&self.handle, "call finished (data received)");
+        }
     }
 
     fn error(&self, error: CallError) {
-        let mut state = self.state.lock().unwrap();
-        state.result = Some(Err(error));
-    }
-}
-
-impl Drop for Call {
-    fn drop(&mut self) {
+        {
+            let mut state = self.state.lock().unwrap();
+            state.result = Some(Err(error));
+        }
         super::drop(self.handle);
-        crate::engine::BusEngine::remove(&self.handle);
+        crate::engine::BusEngine::remove(&self.handle, "call has failed");
+    }
+
+    fn wapm(&self) -> &str {
+        self.wapm.as_ref()
+    }
+
+    fn topic(&self) -> &str {
+        self.topic.as_ref()
     }
 }
 
@@ -63,6 +86,18 @@ impl Call {
     pub fn id(&self) -> u32 {
         self.handle.id
     }
+
+    pub fn handle(&self) -> CallHandle {
+        self.handle.clone()
+    }
+
+    pub fn wapm(&self) -> Cow<'static, str> {
+        self.wapm.clone()
+    }
+
+    pub fn session(&self) -> Option<&str> {
+        self.session.as_ref().map(|a| a.as_str())
+    }
 }
 
 #[derive(Derivative)]
@@ -85,20 +120,49 @@ impl CallBuilder {
 impl CallBuilder {
     /// Upon receiving a particular message from the service that is
     /// invoked this callback will take some action
-    pub fn callback<C, F>(&mut self, callback: F) -> &mut Self
+    pub fn callback<C, F>(mut self, format: SerializationFormat, callback: F) -> Self
     where
         C: Serialize + de::DeserializeOwned + Send + Sync + 'static,
         F: FnMut(C),
         F: Send + 'static,
     {
-        self.call.as_mut().unwrap().callback(callback);
+        self.call.as_mut().unwrap().callback(format, callback);
         self
     }
 
+    // Completes the call but leaves the resources present
+    // for the duration of the life of 'DetachedCall'
+    pub async fn detach<T>(mut self) -> Result<DetachedCall<T>, CallError>
+    where
+        T: de::DeserializeOwned,
+    {
+        let call = self.call.take().unwrap();
+        call.drop_on_data.store(false, Ordering::Release);
+        self.invoke_internal(&call);
+
+        let handle = call.handle.clone();
+        let wapm = call.wapm.clone();
+        let session = call.session.clone();
+
+        let result = call.join::<T>().await?;
+
+        Ok(DetachedCall {
+            handle,
+            result,
+            wapm,
+            session,
+        })
+    }
+
     /// Invokes the call with the specified callbacks
     #[cfg(target_arch = "wasm32")]
     pub fn invoke(mut self) -> Call {
         let call = self.call.take().unwrap();
+        self.invoke_internal(&call);
+        call
+    }
+
+    fn invoke_internal(&self, call: &Call) {
         match &self.request {
             Data::Success(req) => {
                 crate::abi::syscall::call(
@@ -113,8 +177,6 @@ impl CallBuilder {
                 crate::engine::BusEngine::error(call.handle, err.clone());
             }
         }
-
-        call
     }
 
     #[cfg(not(target_arch = "wasm32"))]
@@ -127,7 +189,7 @@ impl Drop for CallBuilder {
     fn drop(&mut self) {
         if let Some(call) = self.call.take() {
             super::drop(call.handle);
-            crate::engine::BusEngine::remove(&call.handle);
+            crate::engine::BusEngine::remove(&call.handle, "call building was dropped");
         }
     }
 }
@@ -136,11 +198,17 @@ impl Call {
     /// Creates another call relative to this call
     /// This can be useful for creating contextual objects using thread calls
     /// and then passing data or commands back and forth to it
-    pub fn call<T>(&self, req: T) -> CallBuilder
+    pub fn call<T>(&self, format: SerializationFormat, req: T) -> CallBuilder
     where
         T: Serialize,
     {
-        super::call_internal(Some(self.handle), self.wapm.clone(), req)
+        super::call_ext(
+            Some(self.handle),
+            self.wapm.clone(),
+            format,
+            self.session.clone(),
+            req,
+        )
     }
 
     /// Upon receiving a particular message from the service that is
@@ -148,7 +216,7 @@ impl Call {
     ///
     /// Note: This must be called before the invoke or things will go wrong
     /// hence there is a builder that invokes this in the right order
-    fn callback<C, F>(&mut self, mut callback: F) -> &mut Self
+    fn callback<C, F>(&self, format: SerializationFormat, mut callback: F) -> &Self
     where
         C: Serialize + de::DeserializeOwned + Send + Sync + 'static,
         F: FnMut(C),
@@ -158,8 +226,9 @@ impl Call {
             callback(req);
             Ok(())
         };
-        let recv = super::callback_internal(self.handle, callback);
-        self.callbacks.lock().unwrap().push(recv);
+        let recv = super::callback_internal(self.handle, format, callback);
+        let mut state = self.state.lock().unwrap();
+        state.callbacks.push(recv);
         self
     }
 
@@ -172,6 +241,43 @@ impl Call {
     }
 }
 
+#[derive(Debug)]
+pub struct DetachedCall<T> {
+    handle: CallHandle,
+    result: T,
+    wapm: Cow<'static, str>,
+    session: Option<String>,
+}
+
+impl<T> DetachedCall<T> {
+    pub fn wapm(&self) -> Cow<'static, str> {
+        self.wapm.clone()
+    }
+
+    pub fn session(&self) -> Option<&str> {
+        self.session.as_ref().map(|a| a.as_str())
+    }
+
+    pub fn handle(&self) -> CallHandle {
+        self.handle.clone()
+    }
+}
+
+impl<T> Drop for DetachedCall<T> {
+    fn drop(&mut self) {
+        super::drop(self.handle);
+        crate::engine::BusEngine::remove(&self.handle, "detached call was dropped");
+    }
+}
+
+impl<T> Deref for DetachedCall<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.result
+    }
+}
+
 #[derive(Debug, Clone)]
 #[must_use = "this `Call` only does something when you consume it"]
 pub struct CallJoin<T>
@@ -222,8 +328,12 @@ where
 
         match response {
             Some(Ok(res)) => {
-                let res = bincode::deserialize::<T>(res.as_ref())
-                    .map_err(|_err| CallError::DeserializationFailed);
+                let res = match self.call.format {
+                    SerializationFormat::Bincode => bincode::deserialize::<T>(res.as_ref())
+                        .map_err(|_err| CallError::DeserializationFailed),
+                    SerializationFormat::Json => serde_json::from_slice(res.as_ref())
+                        .map_err(|_err| CallError::DeserializationFailed),
+                };
                 match res {
                     Ok(data) => Ok(Some(data)),
                     Err(err) => Err(err),
@@ -249,8 +359,12 @@ where
 
         match response {
             Some(Ok(response)) => {
-                let res = bincode::deserialize::<T>(response.as_ref())
-                    .map_err(|_err| CallError::DeserializationFailed);
+                let res = match self.call.format {
+                    SerializationFormat::Bincode => bincode::deserialize::<T>(response.as_ref())
+                        .map_err(|_err| CallError::DeserializationFailed),
+                    SerializationFormat::Json => serde_json::from_slice(response.as_ref())
+                        .map_err(|_err| CallError::DeserializationFailed),
+                };
                 match res {
                     Ok(data) => Poll::Ready(Ok(data)),
                     Err(err) => Poll::Ready(Err(err)),
