@@ -1,3 +1,5 @@
+use std::fmt;
+
 #[repr(C)]
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct CallHandle {
@@ -15,3 +17,9 @@ impl Into<u32> for CallHandle {
         self.id
     }
 }
+
+impl fmt::Display for CallHandle {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "handle_id={}", self.id)
+    }
+}
